/*
 * Copyright (c) 2002-2009, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.directory.business;

import fr.paris.lutece.plugins.directory.utils.DirectoryUtils;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * This class provides Data Access methods for Entry objects
 */
public final class EntryDAO implements IEntryDAO
{
    // Constants
    private static final String SQL_QUERY_NEW_PK = "SELECT MAX( id_entry ) FROM directory_entry";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT ent.id_type,typ.title_key,typ.is_group," +
        "typ.is_comment,typ.class_name,ent.id_entry,ent.id_directory,directory.title,ent.id_entry_parent,ent.title," +
        "ent.help_message,ent.help_message_search,ent.entry_comment,ent.is_mandatory,ent.is_indexed," +
        "ent.is_shown_in_search,ent.is_shown_in_result_list,ent.is_shown_in_result_record,ent.is_fields_in_line,ent.entry_position," +
        "ent.display_width,ent.display_height,ent.is_role_associated,ent.is_workgroup_associated,ent.is_multiple_search_fields,ent.is_shown_in_history,ent.id_entry_associate,ent.request_sql,ent.is_add_value_search_all,ent.label_value_search_all " +
        "FROM directory_entry ent,directory_entry_type typ,directory_directory directory WHERE ent.id_entry = ? and ent.id_type=typ.id_type and " +
        "ent.id_directory=directory.id_directory";
    private static final String SQL_QUERY_INSERT = "INSERT INTO directory_entry ( " +
        "id_entry,id_entry_parent,id_directory,id_type,title,help_message,help_message_search,entry_comment,is_mandatory," +
        "is_indexed,is_shown_in_search,is_shown_in_result_list,is_shown_in_result_record,is_fields_in_line,entry_position,display_width,display_height " +
        ",is_role_associated,is_workgroup_associated,is_multiple_search_fields,is_shown_in_history,id_entry_associate,request_sql,is_add_value_search_all,label_value_search_all )VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
    private static final String SQL_QUERY_DELETE = "DELETE FROM directory_entry WHERE id_entry = ? ";
    private static final String SQL_QUERY_UPDATE = "UPDATE  directory_entry SET " +
        "id_entry=?,id_entry_parent=?,id_directory=?,id_type=?,title=?,help_message=?,help_message_search=?," +
        "entry_comment=?,is_mandatory=?,is_indexed=?,is_shown_in_search=?,is_shown_in_result_list=?," +
        "is_shown_in_result_record=?,is_fields_in_line=?,entry_position=? ,display_width=?,display_height=?," +
        "is_role_associated=?,is_workgroup_associated=?,is_multiple_search_fields=?,is_shown_in_history=?,id_entry_associate=?,request_sql=?,is_add_value_search_all=?,label_value_search_all=? WHERE id_entry=?";
    private static final String SQL_QUERY_SELECT_ENTRY_BY_FILTER = "SELECT ent.id_type,typ.title_key,typ.is_group," +
        "typ.is_comment,typ.class_name,ent.id_entry,ent.id_directory," +
        "ent.id_entry_parent,ent.title,ent.help_message,ent.help_message_search," +
        "ent.entry_comment,ent.is_mandatory,ent.is_indexed,ent.is_shown_in_search,ent.is_shown_in_result_list,ent.is_shown_in_result_record," +
        "ent.is_fields_in_line,ent.entry_position,ent.display_width,ent.display_height,ent.is_role_associated,ent.is_workgroup_associated, " +
        "ent.is_multiple_search_fields,ent.is_shown_in_history,ent.id_entry_associate ,ent.request_sql,ent.is_add_value_search_all,ent.label_value_search_all " +
        "FROM directory_entry ent,directory_entry_type typ  ";
    private static final String SQL_QUERY_SELECT_NUMBER_ENTRY_BY_FILTER = "SELECT COUNT(ent.id_entry) " +
        "FROM directory_entry ent,directory_entry_type typ ";
    private static final String SQL_QUERY_NEW_POSITION = "SELECT MAX(entry_position) " +
        "FROM directory_entry WHERE id_directory = ? ";
    private static final String SQL_FILTER_ID_DIRECTORY = "  ent.id_directory = ? ";
    private static final String SQL_FILTER_POSITION = "  ent.entry_position = ? ";
    private static final String SQL_FILTER_ID_PARENT = " ent.id_entry_parent = ? ";
    private static final String SQL_FILTER_ID_PARENT_IS_NULL = "  ent.id_entry_parent IS NULL ";
    private static final String SQL_FILTER_IS_GROUP = "  typ.is_group = ? ";
    private static final String SQL_FILTER_IS_COMMENT = " typ.is_comment = ? ";
    private static final String SQL_FILTER_IS_SHOWN_IN_RESULT_LIST = "  ent.is_shown_in_result_list=?";
    private static final String SQL_FILTER_IS_SHOWN_IN_HISTORY = "  ent.is_shown_in_history=?";
    private static final String SQL_FILTER_IS_INDEXED = "  ent.is_indexed=?";
    private static final String SQL_FILTER_IS_ROLE_ASSOCIATED = "  ent.is_role_associated=?";
    private static final String SQL_FILTER_IS_WORKGROUP_ASSOCIATED = "  ent.is_workgroup_associated=?";
    private static final String SQL_FILTER_ASSOCIATION_ON_ID_TYPE = " ent.id_type=typ.id_type";
    private static final String SQL_ORDER_BY_POSITION = " ORDER BY ent.entry_position ";
    private static final String SQL_FILTER_ID_TYPE = "  typ.id_type = ? ";
    private static final String SQL_FILTER_ID_ENTRY_ASSOCIATE = "  ent.id_entry_associate = ? ";

    /**
     * Generates a new primary key
     *
     * @param plugin the plugin
     * @return The new primary key
     */
    private int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            // if the table is empty
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Generates a new entry position
     * @param plugin the plugin
     * @return the new entry position
     */
    private int newPosition( int nIdDirectory, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_POSITION, plugin );
        daoUtil.setInt( 1, nIdDirectory );
        daoUtil.executeQuery(  );

        int nPos;

        if ( !daoUtil.next(  ) )
        {
            // if the table is empty
            nPos = 1;
        }

        nPos = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nPos;
    }

    /**
     * Insert a new record in the table.
     *
     * @param entry instance of the Entry object to insert
     * @param plugin the plugin
     * @return the id of the new entry
     */
    public synchronized int insert( IEntry entry, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );

        if ( entry.getParent(  ) != null )
        {
            daoUtil.setInt( 2, entry.getParent(  ).getIdEntry(  ) );
        }
        else
        {
            daoUtil.setIntNull( 2 );
        }

        daoUtil.setInt( 3, entry.getDirectory(  ).getIdDirectory(  ) );
        daoUtil.setInt( 4, entry.getEntryType(  ).getIdType(  ) );
        daoUtil.setString( 5, entry.getTitle(  ) );
        daoUtil.setString( 6, entry.getHelpMessage(  ) );
        daoUtil.setString( 7, entry.getHelpMessageSearch(  ) );
        daoUtil.setString( 8, entry.getComment(  ) );
        daoUtil.setBoolean( 9, entry.isMandatory(  ) );
        daoUtil.setBoolean( 10, entry.isIndexed(  ) );
        daoUtil.setBoolean( 11, entry.isShownInAdvancedSearch(  ) );
        daoUtil.setBoolean( 12, entry.isShownInResultList(  ) );
        daoUtil.setBoolean( 13, entry.isShownInResultRecord(  ) );
        daoUtil.setBoolean( 14, entry.isFieldInLine(  ) );
        daoUtil.setInt( 15, newPosition( entry.getDirectory(  ).getIdDirectory(  ), plugin ) );
        daoUtil.setInt( 16, entry.getDisplayWidth(  ) );
        daoUtil.setInt( 17, entry.getDisplayHeight(  ) );
        daoUtil.setBoolean( 18, entry.isRoleAssociated(  ) );
        daoUtil.setBoolean( 19, entry.isWorkgroupAssociated(  ) );
        daoUtil.setBoolean( 20, entry.isMultipleSearchFields(  ) );
        daoUtil.setBoolean( 21, entry.isShownInHistory(  ) );
        daoUtil.setInt( 22, entry.getEntryAssociate(  ) );
        daoUtil.setString( 23, entry.getRequestSQL(  ) );
        daoUtil.setBoolean( 24, entry.isAddValueAllSearch(  ) );
        daoUtil.setString( 25, entry.getLabelValueAllSearch(  ) );

        entry.setIdEntry( newPrimaryKey( plugin ) );
        daoUtil.setInt( 1, entry.getIdEntry(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return entry.getIdEntry(  );
    }

    /**
     * Load the data of the entry from the table
     *
     * @param nId The identifier of the entry
     * @param plugin the plugin
     * @return the instance of the Entry
     */
    public IEntry load( int nId, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nId );
        daoUtil.executeQuery(  );

        boolean bException = false;
        IEntry entry = null;
        EntryType entryType = null;
        IEntry entryParent = null;
        Directory directory = null;

        if ( daoUtil.next(  ) )
        {
            entryType = new EntryType(  );
            entryType.setIdType( daoUtil.getInt( 1 ) );
            entryType.setTitleI18nKey( daoUtil.getString( 2 ) );
            entryType.setGroup( daoUtil.getBoolean( 3 ) );
            entryType.setComment( daoUtil.getBoolean( 4 ) );
            entryType.setClassName( daoUtil.getString( 5 ) );

            try
            {
                entry = (IEntry) Class.forName( entryType.getClassName(  ) ).newInstance(  );
            }
            catch ( ClassNotFoundException e )
            {
                //  class doesn't exist
                AppLogService.error( e );
                bException = true;
            }
            catch ( InstantiationException e )
            {
                // Class is abstract or is an  interface or haven't accessible builder
                AppLogService.error( e );
                bException = true;
            }
            catch ( IllegalAccessException e )
            {
                // can't access to rhe class
                AppLogService.error( e );
                bException = true;
            }

            if ( bException )
            {
                daoUtil.free(  );

                return null;
            }

            entry.setEntryType( entryType );
            entry.setIdEntry( daoUtil.getInt( 6 ) );
            // insert directory
            directory = new Directory(  );
            directory.setIdDirectory( daoUtil.getInt( 7 ) );
            directory.setTitle( daoUtil.getString( 8 ) );
            entry.setDirectory( directory );

            if ( daoUtil.getObject( 9 ) != null )
            {
                entryParent = new Entry(  );
                entryParent.setIdEntry( daoUtil.getInt( 9 ) );
                entry.setParent( entryParent );
            }

            entry.setTitle( daoUtil.getString( 10 ) );
            entry.setHelpMessage( daoUtil.getString( 11 ) );
            entry.setHelpMessageSearch( daoUtil.getString( 12 ) );
            entry.setComment( daoUtil.getString( 13 ) );
            entry.setMandatory( daoUtil.getBoolean( 14 ) );
            entry.setIndexed( daoUtil.getBoolean( 15 ) );
            entry.setShownInAdvancedSearch( daoUtil.getBoolean( 16 ) );
            entry.setShownInResultList( daoUtil.getBoolean( 17 ) );
            entry.setShownInResultRecord( daoUtil.getBoolean( 18 ) );
            entry.setFieldInLine( daoUtil.getBoolean( 19 ) );
            entry.setPosition( daoUtil.getInt( 20 ) );
            entry.setDisplayWidth( daoUtil.getInt( 21 ) );
            entry.setDisplayHeight( daoUtil.getInt( 22 ) );
            entry.setRoleAssociated( daoUtil.getBoolean( 23 ) );
            entry.setWorkgroupAssociated( daoUtil.getBoolean( 24 ) );
            entry.setMultipleSearchFields( daoUtil.getBoolean( 25 ) );
            entry.setShownInHistory( daoUtil.getBoolean( 26 ) );
            entry.setEntryAssociate( daoUtil.getInt( 27 ) );
            entry.setRequestSQL( daoUtil.getString( 28 ) );
            entry.setAddValueAllSearch( daoUtil.getBoolean( 29 ) );
            entry.setLabelValueAllSearch( daoUtil.getString( 30 ) );
        }

        daoUtil.free(  );

        return entry;
    }

    /**
     * Delete a record from the table
     *
     * @param nIdEntry The identifier of the entry
     * @param plugin the plugin
     */
    public void delete( int nIdEntry, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, nIdEntry );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Update the entry in the table
     *
     * @param entry instance of the Entry object to update
     * @param plugin the plugin
     */
    public void store( IEntry entry, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setInt( 1, entry.getIdEntry(  ) );

        if ( entry.getParent(  ) != null )
        {
            daoUtil.setInt( 2, entry.getParent(  ).getIdEntry(  ) );
        }
        else
        {
            daoUtil.setIntNull( 2 );
        }

        daoUtil.setInt( 3, entry.getDirectory(  ).getIdDirectory(  ) );
        daoUtil.setInt( 4, entry.getEntryType(  ).getIdType(  ) );
        daoUtil.setString( 5, entry.getTitle(  ) );
        daoUtil.setString( 6, entry.getHelpMessage(  ) );
        daoUtil.setString( 7, entry.getHelpMessageSearch(  ) );
        daoUtil.setString( 8, entry.getComment(  ) );
        daoUtil.setBoolean( 9, entry.isMandatory(  ) );
        daoUtil.setBoolean( 10, entry.isIndexed(  ) );
        daoUtil.setBoolean( 11, entry.isShownInAdvancedSearch(  ) );
        daoUtil.setBoolean( 12, entry.isShownInResultList(  ) );
        daoUtil.setBoolean( 13, entry.isShownInResultRecord(  ) );
        daoUtil.setBoolean( 14, entry.isFieldInLine(  ) );
        daoUtil.setInt( 15, entry.getPosition(  ) );
        daoUtil.setInt( 16, entry.getDisplayWidth(  ) );
        daoUtil.setInt( 17, entry.getDisplayHeight(  ) );
        daoUtil.setBoolean( 18, entry.isRoleAssociated(  ) );
        daoUtil.setBoolean( 19, entry.isWorkgroupAssociated(  ) );
        daoUtil.setBoolean( 20, entry.isMultipleSearchFields(  ) );
        daoUtil.setBoolean( 21, entry.isShownInHistory(  ) );
        daoUtil.setInt( 22, entry.getEntryAssociate(  ) );
        daoUtil.setString( 23, entry.getRequestSQL(  ) );
        daoUtil.setBoolean( 24, entry.isAddValueAllSearch(  ) );
        daoUtil.setString( 25, entry.getLabelValueAllSearch(  ) );

        daoUtil.setInt( 26, entry.getIdEntry(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Load the data of all the entry who verify the filter and returns them in a  list
     * @param filter the filter
     * @param plugin the plugin
     * @return  the list of entry
     */
    public List<IEntry> selectEntryListByFilter( EntryFilter filter, Plugin plugin )
    {
        List<IEntry> entryList = new ArrayList<IEntry>(  );
        IEntry entry = null;
        EntryType entryType = null;
        IEntry entryParent = null;
        Directory directory = null;
        boolean bException = false;
        List<String> listStrFilter = new ArrayList<String>(  );

        listStrFilter.add( SQL_FILTER_ASSOCIATION_ON_ID_TYPE );

        if ( filter.containsPosition(  ) )
        {
            listStrFilter.add( SQL_FILTER_POSITION );
        }

        if ( filter.containsIdDirectory(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_DIRECTORY );
        }

        if ( filter.containsIdEntryParent(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_PARENT );
        }

        if ( filter.containsIsEntryParentNull(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_PARENT_IS_NULL );
        }

        if ( filter.containsIsGroup(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_GROUP );
        }

        if ( filter.containsIsShownInResultList(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_SHOWN_IN_RESULT_LIST );
        }

        if ( filter.containsIsShownInHistory(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_SHOWN_IN_HISTORY );
        }

        if ( filter.containsIsIndexed(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_INDEXED );
        }

        if ( filter.containsIsComment(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_COMMENT );
        }

        if ( filter.containsIsWorkgroupAssociated(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_WORKGROUP_ASSOCIATED );
        }

        if ( filter.containsIsRoleAssociated(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_ROLE_ASSOCIATED );
        }

        if ( filter.containsIdType(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_TYPE );
        }

        if ( filter.containsIdEntryAssociate(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_ENTRY_ASSOCIATE );
        }

        String strSQL = DirectoryUtils.buildRequetteWithFilter( SQL_QUERY_SELECT_ENTRY_BY_FILTER, listStrFilter,
                SQL_ORDER_BY_POSITION );
        DAOUtil daoUtil = new DAOUtil( strSQL, plugin );
        int nIndex = 1;

        if ( filter.containsPosition(  ) )
        {
            daoUtil.setInt( nIndex, filter.getPosition(  ) );
            nIndex++;
        }

        if ( filter.containsIdDirectory(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdDirectory(  ) );
            nIndex++;
        }

        if ( filter.containsIdEntryParent(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdEntryParent(  ) );
            nIndex++;
        }

        if ( filter.containsIsGroup(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsGroup(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsShownInResultList(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsShownInResultList(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsShownInHistory(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsShownInHistory(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsIndexed(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsIndexed(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsComment(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsComment(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsWorkgroupAssociated(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsWorkgroupAssociated(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsRoleAssociated(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsRoleAssociated(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIdType(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdType(  ) );
            nIndex++;
        }

        if ( filter.containsIdEntryAssociate(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdEntryAssociate(  ) );
            nIndex++;
        }

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            entryType = new EntryType(  );
            entryType.setIdType( daoUtil.getInt( 1 ) );
            entryType.setTitleI18nKey( daoUtil.getString( 2 ) );
            entryType.setGroup( daoUtil.getBoolean( 3 ) );
            entryType.setComment( daoUtil.getBoolean( 4 ) );
            entryType.setClassName( daoUtil.getString( 5 ) );

            try
            {
                entry = (IEntry) Class.forName( entryType.getClassName(  ) ).newInstance(  );
            }
            catch ( ClassNotFoundException e )
            {
                //  class doesn't exist
                AppLogService.error( e );
                bException = true;
            }
            catch ( InstantiationException e )
            {
                // Class is abstract or is an  interface or haven't accessible builder
                AppLogService.error( e );
                bException = true;
            }
            catch ( IllegalAccessException e )
            {
                // can't access to rhe class
                AppLogService.error( e );
                bException = true;
            }

            if ( bException )
            {
                daoUtil.free(  );

                return null;
            }

            entry.setEntryType( entryType );
            entry.setIdEntry( daoUtil.getInt( 6 ) );
            // insert form
            directory = new Directory(  );
            directory.setIdDirectory( daoUtil.getInt( 7 ) );
            entry.setDirectory( directory );

            if ( daoUtil.getObject( 8 ) != null )
            {
                entryParent = new Entry(  );
                entryParent.setIdEntry( daoUtil.getInt( 8 ) );
                entry.setParent( entryParent );
            }

            entry.setTitle( daoUtil.getString( 9 ) );
            entry.setHelpMessage( daoUtil.getString( 10 ) );
            entry.setHelpMessageSearch( daoUtil.getString( 11 ) );
            entry.setComment( daoUtil.getString( 12 ) );
            entry.setMandatory( daoUtil.getBoolean( 13 ) );
            entry.setIndexed( daoUtil.getBoolean( 14 ) );
            entry.setShownInAdvancedSearch( daoUtil.getBoolean( 15 ) );
            entry.setShownInResultList( daoUtil.getBoolean( 16 ) );
            entry.setShownInResultRecord( daoUtil.getBoolean( 17 ) );
            entry.setFieldInLine( daoUtil.getBoolean( 18 ) );
            entry.setPosition( daoUtil.getInt( 19 ) );
            entry.setDisplayWidth( daoUtil.getInt( 20 ) );
            entry.setDisplayHeight( daoUtil.getInt( 21 ) );
            entry.setRoleAssociated( daoUtil.getBoolean( 22 ) );
            entry.setWorkgroupAssociated( daoUtil.getBoolean( 23 ) );
            entry.setMultipleSearchFields( daoUtil.getBoolean( 24 ) );
            entry.setShownInHistory( daoUtil.getBoolean( 25 ) );
            entry.setEntryAssociate( daoUtil.getInt( 26 ) );
            entry.setRequestSQL( daoUtil.getString( 27 ) );
            entry.setAddValueAllSearch( daoUtil.getBoolean( 28 ) );
            entry.setLabelValueAllSearch( daoUtil.getString( 29 ) );

            entryList.add( entry );
        }

        daoUtil.free(  );

        return entryList;
    }

    /**
     * Return  the number of entry who verify the filter
     * @param filter the filter
     * @param plugin the plugin
     * @return   the number of entry who verify the filter
     */
    public int selectNumberEntryByFilter( EntryFilter filter, Plugin plugin )
    {
        int nNumberEntry = 0;
        List<String> listStrFilter = new ArrayList<String>(  );

        listStrFilter.add( SQL_FILTER_ASSOCIATION_ON_ID_TYPE );

        if ( filter.containsIdDirectory(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_DIRECTORY );
        }

        if ( filter.containsIdEntryParent(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_PARENT );
        }

        if ( filter.containsIsEntryParentNull(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_PARENT_IS_NULL );
        }

        if ( filter.containsIsGroup(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_GROUP );
        }

        if ( filter.containsIsShownInResultList(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_SHOWN_IN_RESULT_LIST );
        }

        if ( filter.containsIsIndexed(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_INDEXED );
        }

        if ( filter.containsIsComment(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_COMMENT );
        }

        if ( filter.containsIsWorkgroupAssociated(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_WORKGROUP_ASSOCIATED );
        }

        if ( filter.containsIsRoleAssociated(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_ROLE_ASSOCIATED );
        }

        if ( filter.containsIdType(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_TYPE );
        }

        if ( filter.containsIdEntryAssociate(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_ENTRY_ASSOCIATE );
        }

        String strSQL = DirectoryUtils.buildRequetteWithFilter( SQL_QUERY_SELECT_NUMBER_ENTRY_BY_FILTER, listStrFilter,
                null );

        DAOUtil daoUtil = new DAOUtil( strSQL, plugin );
        int nIndex = 1;

        if ( filter.containsIdDirectory(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdDirectory(  ) );
            nIndex++;
        }

        if ( filter.containsIdEntryParent(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdEntryParent(  ) );
            nIndex++;
        }

        if ( filter.containsIsGroup(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsGroup(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsShownInResultList(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsShownInResultList(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsIndexed(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsIndexed(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsComment(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsComment(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsWorkgroupAssociated(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsWorkgroupAssociated(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIsRoleAssociated(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.getIsRoleAssociated(  ) == EntryFilter.FILTER_TRUE );
            nIndex++;
        }

        if ( filter.containsIdType(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdType(  ) );
            nIndex++;
        }

        if ( filter.containsIdEntryAssociate(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdEntryAssociate(  ) );
            nIndex++;
        }

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            nNumberEntry = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return nNumberEntry;
    }
}
