/*
 * Copyright (c) 2002-2009, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.directory.business;

import fr.paris.lutece.plugins.directory.utils.DirectoryUtils;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.workgroup.AdminWorkgroupService;
import fr.paris.lutece.util.ReferenceItem;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * This class provides Data Access methods for record objects
 */
public final class RecordDAO implements IRecordDAO
{
    // Constants
    private static final String SQL_QUERY_NEW_PK = "SELECT MAX( id_record ) FROM directory_record";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT id_record,date_creation,id_directory,is_enabled,role_key,workgroup_key,x,y " +
        "FROM directory_record WHERE id_record=? ";
    private static final String SQL_QUERY_FIND_BY_LIST_PRIMARY_KEY = "SELECT id_record,date_creation,id_directory,is_enabled,role_key,workgroup_key,x,y " +
        "FROM directory_record WHERE id_record IN ( ?";
    private static final String SQL_QUERY_INSERT = "INSERT INTO directory_record ( " +
        "id_record,date_creation,id_directory,is_enabled,role_key,workgroup_key,x,y ) VALUES(?,?,?,?,?,?,?,?)";
    private static final String SQL_QUERY_DELETE = "DELETE FROM directory_record WHERE id_record = ? ";
    private static final String SQL_QUERY_DELETE_BY_ID_DIRECTORY = "DELETE FROM directory_record WHERE id_directory = ?";
    private static final String SQL_QUERY_UPDATE = "UPDATE directory_record SET " +
        "id_record=?,date_creation=?,id_directory=?,is_enabled=?,role_key=?,workgroup_key=?,x=?,y=? WHERE id_record=?";
    private static final String SQL_QUERY_SELECT_RECORD_BY_FILTER = "SELECT id_record,date_creation,id_directory,is_enabled,role_key,workgroup_key " +
        "FROM directory_record  ";
    private static final String SQL_QUERY_SELECT_RECORD_ID_BY_FILTER = "SELECT id_record FROM directory_record ";
    private static final String SQL_QUERY_SELECT_COUNT_BY_FILTER = "SELECT COUNT(id_record) " +
        "FROM directory_record  ";
    private static final String SQL_QUERY_SELECT_DIRECTORY_ID = "SELECT id_directory FROM directory_record WHERE id_record=?";
    private static final String SQL_QUERY_SELECT_COUNT_DIRECYTORY_RECORD_HAS_WORKFLOW = "SELECT COUNT(*) FROM directory_record WHERE id_directory = ? AND workgroup_key IS NOT NULL";
    private static final String SQL_FILTER_ID_DIRECTORY = "	 id_directory = ? ";
    private static final String SQL_FILTER_IS_ENABLED = " is_enabled = ? ";
    private static final String SQL_FILTER_OR = " OR ";
    private static final String SQL_FILTER_OPEN_PARENTHESIS = " ( ";
    private static final String SQL_FILTER_CLOSE_PARENTHESIS = " ) ";
    private static final String SQL_FILTER_WORKGROUP_LIST = " workgroup_key IN ( ? ";
    private static final String SQL_FILTER_ROLE_LIST = " role_key IN ( ? ";
    private static final String SQL_ADITIONAL_PARAMETER = ",?";
    private static final String SQL_FILTER_WORKGROUP_IS_NULL = " workgroup_key IS NULL ";
    private static final String SQL_FILTER_ROLE_IS_NULL = " role_key IS NULL ";
    private static final String SQL_ORDER_BY_DATE_CREATION_ASC = " ORDER BY date_creation ASC ";

    /**
     * Generates a new primary key
     *
     * @param plugin the plugin
     * @return The new primary key
     */
    private int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            // if the table is empty
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Insert a new record in the table.
     * @param record instance of the record  object to insert
     * @param plugin the plugin
     * @return the id of the record
     */
    public synchronized int insert( Record record, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );

        daoUtil.setTimestamp( 2, record.getDateCreation(  ) );
        daoUtil.setInt( 3, record.getDirectory(  ).getIdDirectory(  ) );
        daoUtil.setBoolean( 4, record.isEnabled(  ) );
        daoUtil.setString( 5, record.getRoleKey(  ) );
        daoUtil.setString( 6, record.getWorkgroup(  ) );
        daoUtil.setDouble( 7, record.getX(  ) );
        daoUtil.setDouble( 8, record.getY(  ) );

        record.setIdRecord( newPrimaryKey( plugin ) );
        daoUtil.setInt( 1, record.getIdRecord(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return record.getIdRecord(  );
    }

    /**
     * Load the data of the record from the table
     *
     * @param nIdRecord The identifier of the record
     * @param plugin the plugin
     * @return the instance of the record
     */
    public Record load( int nIdRecord, Plugin plugin )
    {
        Record record = null;
        Directory directory;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nIdRecord );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            record = new Record(  );
            record.setIdRecord( daoUtil.getInt( 1 ) );
            record.setDateCreation( daoUtil.getTimestamp( 2 ) );
            directory = new Directory(  );
            directory.setIdDirectory( daoUtil.getInt( 3 ) );
            record.setDirectory( directory );
            record.setEnabled( daoUtil.getBoolean( 4 ) );
            record.setRoleKey( daoUtil.getString( 5 ) );
            record.setWorkgroup( daoUtil.getString( 6 ) );
            record.setX( daoUtil.getDouble( 7 ) );
            record.setY( daoUtil.getDouble( 8 ) );
        }

        daoUtil.free(  );

        return record;
    }

    /**
     * Test if directory one or more directory record has a workflow
     * @param nIdDirectory
     * @param plugin the plugin
     * @return true if one or more record has a worflow
     */
    public Boolean direcytoryRecordListHasWorkflow( int nIdDirectory, Plugin plugin )
    {
        Boolean bResult = null;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_SELECT_COUNT_DIRECYTORY_RECORD_HAS_WORKFLOW, plugin );
        daoUtil.setInt( 1, nIdDirectory );

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            int nNb = daoUtil.getInt( 1 );

            if ( nNb > 0 )
            {
                bResult = Boolean.TRUE;
            }
            else
            {
                bResult = Boolean.FALSE;
            }
        }

        daoUtil.free(  );

        return bResult;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.directory.business.IRecordDAO#loadList(java.util.List, fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public List<Record> loadList( List<Integer> listId, Plugin plugin )
    {
        List<Record> lRecord = new ArrayList<Record>(  );
        int nSize = listId.size(  );

        if ( nSize > 0 )
        {
            Directory directory;

            StringBuffer sb = new StringBuffer( SQL_QUERY_FIND_BY_LIST_PRIMARY_KEY );

            for ( int i = 1; i < nSize; i++ )
            {
                sb.append( SQL_ADITIONAL_PARAMETER );
            }

            sb.append( SQL_FILTER_CLOSE_PARENTHESIS );

            DAOUtil daoUtil = new DAOUtil( sb.toString(  ), plugin );

            for ( int i = 0; i < nSize; i++ )
            {
                daoUtil.setInt( 1 + i, listId.get( i ) );
            }

            daoUtil.executeQuery(  );

            while ( daoUtil.next(  ) )
            {
                Record record = new Record(  );
                record.setIdRecord( daoUtil.getInt( 1 ) );
                record.setDateCreation( daoUtil.getTimestamp( 2 ) );
                directory = new Directory(  );
                directory.setIdDirectory( daoUtil.getInt( 3 ) );
                record.setDirectory( directory );
                record.setEnabled( daoUtil.getBoolean( 4 ) );
                record.setRoleKey( daoUtil.getString( 5 ) );
                record.setWorkgroup( daoUtil.getString( 6 ) );
                record.setX( daoUtil.getDouble( 7 ) );
                record.setY( daoUtil.getDouble( 8 ) );
                lRecord.add( record );
            }

            daoUtil.free(  );
        }

        return lRecord;
    }

    /**
     *  Delete a record from the table
     *
     * @param nIdRecord The identifier of the record
     * @param plugin the plugin
     */
    public void delete( int nIdRecord, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, nIdRecord );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.directory.business.IRecordDAO#deleteRecordByDirectoryId(java.lang.Integer, fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public void deleteRecordByDirectoryId( Integer nDirectoryId, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_BY_ID_DIRECTORY, plugin );
        daoUtil.setInt( 1, nDirectoryId );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Update the record in the table
     *
     * @param record instance of the record object to update
     * @param plugin the plugin
     */
    public void store( Record record, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );
        daoUtil.setInt( 1, record.getIdRecord(  ) );
        daoUtil.setTimestamp( 2, record.getDateCreation(  ) );
        daoUtil.setInt( 3, record.getDirectory(  ).getIdDirectory(  ) );
        daoUtil.setBoolean( 4, record.isEnabled(  ) );
        daoUtil.setString( 5, record.getRoleKey(  ) );
        daoUtil.setString( 6, record.getWorkgroup(  ) );
        daoUtil.setDouble( 7, record.getX(  ) );
        daoUtil.setDouble( 8, record.getY(  ) );
        daoUtil.setInt( 9, record.getIdRecord(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Load the data of all the record who verify the filter and returns them in a  list
     * @param filter the filter
     * @param plugin the plugin
     * @return  the list of record
     */
    public List<Record> selectListByFilter( RecordFieldFilter filter, Plugin plugin )
    {
        List<Record> recordList = new ArrayList<Record>(  );
        Record record;
        Directory directory;

        List<String> listStrFilter = buildFilterQueryHeader( filter );

        String strSQL = DirectoryUtils.buildRequetteWithFilter( SQL_QUERY_SELECT_RECORD_BY_FILTER, listStrFilter,
                SQL_ORDER_BY_DATE_CREATION_ASC );

        DAOUtil daoUtil = new DAOUtil( strSQL, plugin );

        daoUtil = buildFilterQueryFooter( daoUtil, filter, 1 );

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            record = new Record(  );
            record.setIdRecord( daoUtil.getInt( 1 ) );
            record.setDateCreation( daoUtil.getTimestamp( 2 ) );
            directory = new Directory(  );
            directory.setIdDirectory( daoUtil.getInt( 3 ) );
            record.setDirectory( directory );
            record.setEnabled( daoUtil.getBoolean( 4 ) );
            record.setRoleKey( daoUtil.getString( 5 ) );
            record.setWorkgroup( daoUtil.getString( 6 ) );

            recordList.add( record );
        }

        daoUtil.free(  );

        return recordList;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.directory.business.IRecordDAO#getDirectoryIdByRecordId(java.lang.Integer, fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public Integer getDirectoryIdByRecordId( Integer nRecordId, Plugin plugin )
    {
        Integer nResult = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_SELECT_DIRECTORY_ID, plugin );

        daoUtil.setInt( 1, nRecordId );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            nResult = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return nResult;
    }

    /**
     * Load the data of all the record who verify the filter and returns them in a  list
     * @param filter the filter
     * @param plugin the plugin
     * @return  the list of record
     */
    public int selectCountByFilter( RecordFieldFilter filter, Plugin plugin )
    {
        int nIdCount = 0;
        List<String> listStrFilter = buildFilterQueryHeader( filter );

        String strSQL = DirectoryUtils.buildRequetteWithFilter( SQL_QUERY_SELECT_COUNT_BY_FILTER, listStrFilter, null );

        DAOUtil daoUtil = new DAOUtil( strSQL, plugin );

        daoUtil = buildFilterQueryFooter( daoUtil, filter, 1 );

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            nIdCount = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return nIdCount;
    }

    /**
     * Load the data of all the record id who verify the filter and returns them in a  list
     * @param filter the filter
     * @param plugin the plugin
     * @return  the list of id record
     */
    public List<Integer> selectListIdByFilter( RecordFieldFilter filter, Plugin plugin )
    {
        List<Integer> recordList = new ArrayList<Integer>(  );
        List<String> listStrFilter = buildFilterQueryHeader( filter );

        String strSQL = DirectoryUtils.buildRequetteWithFilter( SQL_QUERY_SELECT_RECORD_ID_BY_FILTER, listStrFilter,
                SQL_ORDER_BY_DATE_CREATION_ASC );

        DAOUtil daoUtil = new DAOUtil( strSQL, plugin );

        daoUtil = buildFilterQueryFooter( daoUtil, filter, 1 );

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            recordList.add( daoUtil.getInt( 1 ) );
        }

        daoUtil.free(  );

        return recordList;
    }

    /**
     * Build beginning SQL query by given filter
     * @param filter the filter to apply
     * @return List of string to add to the SQL query
     */
    private List<String> buildFilterQueryHeader( RecordFieldFilter filter )
    {
        List<String> listStrFilter = new ArrayList<String>(  );

        if ( filter.containsIdDirectory(  ) )
        {
            listStrFilter.add( SQL_FILTER_ID_DIRECTORY );
        }

        if ( filter.containsIsDisabled(  ) )
        {
            listStrFilter.add( SQL_FILTER_IS_ENABLED );
        }

        if ( filter.containsWorkgroupKeyList(  ) )
        {
            StringBuffer sbWorkgroupFilter = new StringBuffer(  );

            boolean bContaintsAll = false;
            List<String> lWorkgroupKey = new ArrayList<String>(  );

            for ( ReferenceItem rWorkgroupKey : filter.getWorkgroupKeyList(  ) )
            {
                //FIXME : Hack for workgroup filter
                if ( AdminWorkgroupService.ALL_GROUPS.equals( rWorkgroupKey.getCode(  ) ) )
                {
                    bContaintsAll = true;
                }
                else
                {
                    lWorkgroupKey.add( rWorkgroupKey.getCode(  ) );
                }
            }

            int nWorkgroupKeySize = lWorkgroupKey.size(  );

            if ( bContaintsAll )
            {
                if ( nWorkgroupKeySize > 0 )
                {
                    sbWorkgroupFilter.append( SQL_FILTER_OPEN_PARENTHESIS + SQL_FILTER_WORKGROUP_IS_NULL +
                        SQL_FILTER_OR );
                }
                else
                {
                    sbWorkgroupFilter.append( SQL_FILTER_WORKGROUP_IS_NULL );
                }
            }

            if ( nWorkgroupKeySize > 0 )
            {
                for ( int i = 0; i < nWorkgroupKeySize; i++ )
                {
                    if ( i < 1 )
                    {
                        sbWorkgroupFilter.append( SQL_FILTER_WORKGROUP_LIST );
                    }
                    else
                    {
                        sbWorkgroupFilter.append( SQL_ADITIONAL_PARAMETER );
                    }
                }

                sbWorkgroupFilter.append( SQL_FILTER_CLOSE_PARENTHESIS );
            }

            if ( bContaintsAll && ( nWorkgroupKeySize > 0 ) )
            {
                sbWorkgroupFilter.append( SQL_FILTER_CLOSE_PARENTHESIS );
            }

            listStrFilter.add( sbWorkgroupFilter.toString(  ) );
        }

        if ( filter.containsRoleKeyList(  ) )
        {
            List<String> lRoleKeyList = filter.getRoleKeyList(  );
            int nSize = lRoleKeyList.size(  );

            StringBuffer sbRoleKeyFilter = new StringBuffer(  );

            if ( filter.includeRoleNull(  ) )
            {
                if ( nSize > 0 )
                {
                    sbRoleKeyFilter.append( SQL_FILTER_OPEN_PARENTHESIS + SQL_FILTER_ROLE_IS_NULL + SQL_FILTER_OR );
                }
                else
                {
                    sbRoleKeyFilter.append( SQL_FILTER_ROLE_IS_NULL );
                }
            }

            if ( nSize > 0 )
            {
                for ( int i = 0; i < nSize; i++ )
                {
                    if ( i < 1 )
                    {
                        sbRoleKeyFilter.append( SQL_FILTER_ROLE_LIST );
                    }
                    else
                    {
                        sbRoleKeyFilter.append( SQL_ADITIONAL_PARAMETER );
                    }
                }

                sbRoleKeyFilter.append( SQL_FILTER_CLOSE_PARENTHESIS );
            }

            if ( filter.includeRoleNull(  ) && ( nSize > 0 ) )
            {
                sbRoleKeyFilter.append( SQL_FILTER_CLOSE_PARENTHESIS );
            }

            listStrFilter.add( sbRoleKeyFilter.toString(  ) );
        }

        return listStrFilter;
    }

    /**
     * Build ending SQL query by given filter
     * @param filter the filter to apply
     * @param nIndex
     * @return List of string to add to the SQL query
     */
    private DAOUtil buildFilterQueryFooter( DAOUtil daoUtil, RecordFieldFilter filter, int nIndex )
    {
        DAOUtil result = daoUtil;

        if ( filter.containsIdDirectory(  ) )
        {
            result.setInt( nIndex, filter.getIdDirectory(  ) );
            nIndex++;
        }

        if ( filter.containsIsDisabled(  ) )
        {
            result.setInt( nIndex, filter.getIsDisabled(  ) );
            nIndex++;
        }

        if ( filter.containsWorkgroupKeyList(  ) )
        {
            for ( ReferenceItem rWorkgroupKey : filter.getWorkgroupKeyList(  ) )
            {
                //FIXME : Hack for workgroup filter
                if ( !AdminWorkgroupService.ALL_GROUPS.equals( rWorkgroupKey.getCode(  ) ) )
                {
                    result.setString( nIndex, rWorkgroupKey.getCode(  ) );
                    nIndex++;
                }
            }
        }

        if ( filter.containsRoleKeyList(  ) )
        {
            for ( String strRole : filter.getRoleKeyList(  ) )
            {
                result.setString( nIndex, strRole );
                nIndex++;
            }
        }

        return result;
    }
}
