/*
 * Copyright (c) 2002-2009, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.directory.service.directorysearch;

import fr.paris.lutece.plugins.directory.business.Directory;
import fr.paris.lutece.plugins.directory.business.IndexerAction;
import fr.paris.lutece.plugins.directory.business.IndexerActionFilter;
import fr.paris.lutece.plugins.directory.business.IndexerActionHome;
import fr.paris.lutece.plugins.directory.business.RecordField;
import fr.paris.lutece.plugins.directory.business.RecordFieldFilter;
import fr.paris.lutece.plugins.directory.business.RecordHome;
import fr.paris.lutece.plugins.directory.service.DirectoryPlugin;
import fr.paris.lutece.plugins.directory.utils.DirectoryUtils;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.store.FSDirectory;

import java.io.IOException;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeSet;


/**
 * DirectorySearchService
 */
public class DirectorySearchService
{
    private static final String BEAN_SEARCH_ENGINE = "directorySearchEngine";
    private static final String PATH_INDEX = "directory.internalIndexer.lucene.indexPath";
    private static final String PROPERTY_WRITER_MERGE_FACTOR = "directory.internalIndexer.lucene.writer.mergeFactor";
    private static final String PROPERTY_WRITER_MAX_FIELD_LENGTH = "directory.internalIndexer.lucene.writer.maxFieldLength";
    private static final String PROPERTY_ANALYSER_CLASS_NAME = "directory.internalIndexer.lucene.analyser.className";
    private static final int DEFAULT_WRITER_MERGE_FACTOR = 20;
    private static final int DEFAULT_WRITER_MAX_FIELD_LENGTH = 1000000;
    private static String _strIndex;
    private static int _nWriterMergeFactor;
    private static int _nWriterMaxFieldLength;
    private static Analyzer _analyzer;
    private static Searcher _searcher;
    private static IDirectorySearchIndexer _indexer;

    // Constants corresponding to the variables defined in the lutece.properties file
    private static DirectorySearchService _singleton;

    /** Creates a new instance of DirectorySearchService */
    private DirectorySearchService(  )
    {
        // Read configuration properties
        _strIndex = AppPathService.getPath( PATH_INDEX );

        if ( ( _strIndex == null ) || ( _strIndex.equals( "" ) ) )
        {
            throw new AppException( "Lucene index path not found in directory.properties", null );
        }

        _nWriterMergeFactor = AppPropertiesService.getPropertyInt( PROPERTY_WRITER_MERGE_FACTOR,
                DEFAULT_WRITER_MERGE_FACTOR );
        _nWriterMaxFieldLength = AppPropertiesService.getPropertyInt( PROPERTY_WRITER_MAX_FIELD_LENGTH,
                DEFAULT_WRITER_MAX_FIELD_LENGTH );

        String strAnalyserClassName = AppPropertiesService.getProperty( PROPERTY_ANALYSER_CLASS_NAME );

        if ( ( strAnalyserClassName == null ) || ( strAnalyserClassName.equals( "" ) ) )
        {
            throw new AppException( "Analyser class name not found in directory.properties", null );
        }

        _indexer = (IDirectorySearchIndexer) SpringContextService.getPluginBean( "directory", "directoryIndexer" );

        try
        {
            _analyzer = (Analyzer) Class.forName( strAnalyserClassName ).newInstance(  );
        }
        catch ( Exception e )
        {
            throw new AppException( "Failed to load Lucene Analyzer class", e );
        }
    }

    /**
     *
     * @return singleton
     */
    public static DirectorySearchService getInstance(  )
    {
        if ( _singleton == null )
        {
            _singleton = new DirectorySearchService(  );
        }

        return _singleton;
    }

    /**
     * Removes from list all the elements that are not contained in the other list
     * Faster than classic "List.retainAll" because each id is unique
     * @param list1 input list 1
     * @param list2 input list 2
     * @return the result list
     */
    private List<Integer> retainAll( List<Integer> list1, List<Integer> list2 )
    {
        List<Integer> lresult = null;

        if ( list1.size(  ) < list2.size(  ) )
        {
            Set<Integer> ts1 = new TreeSet<Integer>( list2 );

            Iterator<Integer> it = list1.iterator(  );

            while ( it.hasNext(  ) )
            {
                if ( !ts1.contains( it.next(  ) ) )
                {
                    it.remove(  );
                }
            }

            lresult = list1;
        }
        else
        {
            Set<Integer> ts1 = new TreeSet<Integer>( list1 );

            Iterator<Integer> it = list2.iterator(  );

            while ( it.hasNext(  ) )
            {
                if ( !ts1.contains( it.next(  ) ) )
                {
                    it.remove(  );
                }
            }

            lresult = list2;
        }

        return lresult;
    }

    /**
     * Return a list of record key return by the search
     * @param directory the directory
     * @param mapSearch a map which contains for each entry the list of recorField(value of field search) associate
     * @param filter the filter
     * @param plugin the plugin
     * @return a list of record key return by the search
     */
    @SuppressWarnings( "unchecked" )
    public List<Integer> getSearchResults( Directory directory, HashMap<String, List<RecordField>> mapSearch,
        RecordFieldFilter filter, Plugin plugin )
    {
        List<Integer> listRecordResult = new ArrayList<Integer>(  );

        listRecordResult = RecordHome.getListRecordId( filter, plugin );

        if ( mapSearch != null )
        {
            List<Integer> listRecordResultTmp;
            List<RecordField> recordFieldSearch;
            HashMap<String, Object> mapSearchItemEntry;
            boolean bSearchEmpty;

            try
            {
                _searcher = new IndexSearcher( _strIndex );

                IDirectorySearchEngine engine = (IDirectorySearchEngine) SpringContextService.getPluginBean( DirectoryPlugin.PLUGIN_NAME,
                        BEAN_SEARCH_ENGINE );

                if ( mapSearch != null )
                {
                    for ( Object entryMapSearch : mapSearch.entrySet(  ) )
                    {
                        listRecordResultTmp = new ArrayList<Integer>(  );
                        recordFieldSearch = ( (Entry<String, List<RecordField>>) entryMapSearch ).getValue(  );

                        int nIdEntry = DirectoryUtils.convertStringToInt( ( (Entry<String, List<RecordField>>) entryMapSearch ).getKey(  ) );
                        bSearchEmpty = true;

                        if ( recordFieldSearch != null )
                        {
                            mapSearchItemEntry = new HashMap<String, Object>(  );

                            for ( RecordField recordField : recordFieldSearch )
                            {
                                recordField.getEntry(  ).addSearchCriteria( mapSearchItemEntry, recordField );
                            }

                            if ( mapSearchItemEntry.size(  ) > 0 )
                            {
                                bSearchEmpty = false;
                                mapSearchItemEntry.put( DirectorySearchItem.FIELD_ID_DIRECTORY,
                                    directory.getIdDirectory(  ) );
                                mapSearchItemEntry.put( DirectorySearchItem.FIELD_ID_DIRECTORY_ENTRY, nIdEntry );
                                listRecordResultTmp.addAll( engine.getSearchResults( mapSearchItemEntry ) );
                            }

                            if ( !bSearchEmpty )
                            {
                                listRecordResult = this.retainAll( listRecordResult, listRecordResultTmp );
                            }
                        }
                    }
                }
            }
            catch ( Exception e )
            {
                AppLogService.error( e.getMessage(  ), e );
                // If an error occurred clean result list
                listRecordResult = new ArrayList<Integer>(  );
            }
            finally
            {
                try
                {
                    if ( _searcher != null )
                    {
                        _searcher.close(  );
                    }
                }
                catch ( Exception e )
                {
                    AppLogService.error( e.getMessage(  ), e );
                }
            }
        }

        return listRecordResult;
    }

    /**
     * Process indexing
     * @param bCreate true for start full indexing
     *                                   false for begin incremental indexing
     * @return the log
     */
    public String processIndexing( boolean bCreate )
    {
        StringBuffer sbLogs = new StringBuffer(  );
        IndexWriter writer = null;
        boolean bCreateIndex = bCreate;

        try
        {
            sbLogs.append( "\r\nIndexing all contents ...\r\n" );

            if ( !IndexReader.indexExists( _strIndex ) )
            { //init index
                bCreateIndex = true;
            }

            if ( !bCreateIndex && IndexReader.isLocked( _strIndex ) )
            {
                IndexReader.unlock( FSDirectory.getDirectory( _strIndex ) );
            }

            writer = new IndexWriter( _strIndex, _analyzer, bCreateIndex );
            writer.setMergeFactor( _nWriterMergeFactor );
            writer.setMaxFieldLength( _nWriterMaxFieldLength );

            Date start = new Date(  );

            sbLogs.append( "\r\n<strong>Indexer : " );
            sbLogs.append( _indexer.getName(  ) );
            sbLogs.append( " - " );
            sbLogs.append( _indexer.getDescription(  ) );
            sbLogs.append( "</strong>\r\n" );
            _indexer.processIndexing( writer, bCreateIndex, sbLogs );

            Date end = new Date(  );

            sbLogs.append( "Duration of the treatment : " );
            sbLogs.append( end.getTime(  ) - start.getTime(  ) );
            sbLogs.append( " milliseconds\r\n" );

            writer.optimize(  );
        }
        catch ( Exception e )
        {
            sbLogs.append( " caught a " );
            sbLogs.append( e.getClass(  ) );
            sbLogs.append( "\n with message: " );
            sbLogs.append( e.getMessage(  ) );
            sbLogs.append( "\r\n" );
            AppLogService.error( "Indexing error : " + e.getMessage(  ), e );
        }
        finally
        {
            try
            {
                if ( writer != null )
                {
                    writer.close(  );
                }
            }
            catch ( IOException e )
            {
                AppLogService.error( e.getMessage(  ), e );
            }
        }

        return sbLogs.toString(  );
    }

    /**
     * Add Indexer Action  to perform on a record
     * @param nIdRecord the id of the record
     * @param nIdTask the key of the action to do
     * @param plugin the plugin
     */
    public void addIndexerAction( int nIdRecord, int nIdTask, Plugin plugin )
    {
        IndexerAction indexerAction = new IndexerAction(  );
        indexerAction.setIdRecord( nIdRecord );
        indexerAction.setIdTask( nIdTask );
        IndexerActionHome.create( indexerAction, plugin );
    }

    /**
     * Remove a Indexer Action
     * @param nIdAction the key of the action to remove
     * @param plugin the plugin
     */
    public void removeIndexerAction( int nIdAction, Plugin plugin )
    {
        IndexerActionHome.remove( nIdAction, plugin );
    }

    /**
     * return a list of IndexerAction by task key
     * @param nIdTask the task kety
     * @param plugin the plugin
     * @return a list of IndexerAction
     */
    public List<IndexerAction> getAllIndexerActionByTask( int nIdTask, Plugin plugin )
    {
        IndexerActionFilter filter = new IndexerActionFilter(  );
        filter.setIdTask( nIdTask );

        return IndexerActionHome.getList( filter, plugin );
    }

    /**
     * return searcher
     * @return searcher
     */
    public Searcher getSearcher(  )
    {
        return _searcher;
    }

    /**
     * set searcher
     * @param searcher searcher
     */
    public void setSearcher( Searcher searcher )
    {
        _searcher = searcher;
    }
}
